/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.mail;

import java.util.Map;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.mail.MailException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;

/**
 * SimpleMailService provides
 *
 * @author Timo Westk&auml;mper
 */
public class SimpleMailService implements MailService{
    
    private static final Logger logger = LoggerFactory.getLogger(SimpleMailService.class);
    
    private JavaMailSender mailSender;
    
    private SimpleMailMessage messageTemplate;
    
    /* (non-Javadoc)
     * @see com.mysema.portal.service.MailService#sendMessage(java.lang.String, java.lang.String, java.lang.String)
     */
    public final void sendMessage(String recipient, String subject, String body) {
        sendMessage(new String[]{recipient}, subject, body);
    }

    /* (non-Javadoc)
     * @see com.mysema.portal.service.MailService#sendMessage(java.lang.String[], java.lang.String, java.lang.String)
     */
    public final void sendMessage(String[] recipients, String subject, String body) {
        SimpleMailMessage  msg = new SimpleMailMessage();
        this.messageTemplate.copyTo(msg);
        msg.setTo(recipients);
        msg.setSubject(subject);
        msg.setText(body);
        
        send(msg);        
    }
   
    public void sendHTMLMessage(String recipient, String subject, String body) {
        sendHTMLMessage(new String[]{recipient}, subject, body);
    }  
    
    public void sendHTMLMessage(String[] recipients, String subject, String body) {
        try {
            MimeMessage message = mailSender.createMimeMessage();           
            MimeMessageHelper helper = new MimeMessageHelper(message, true, "UTF-8");
            helper.setFrom(messageTemplate.getFrom());
            helper.setTo(recipients);
            helper.setSubject(subject);
            helper.setText(body, true);
            mailSender.send(message);
        } catch (MessagingException e) {
            String error = "Caught " + e.getClass().getName();
            logger.error(error, e);
            throw new RuntimeException(error, e);
        }
    }   
    
    /*
     * (non-Javadoc)
     * 
     * @see com.mysema.portal.service.MailService#send(org.springframework.mail.SimpleMailMessage)
     */
    public final void send(SimpleMailMessage msg) {
        try {
            mailSender.send(msg);
        } catch (MailException e) {
            String error = "Caught " + e.getClass().getName();
            logger.error(error, e);
            throw new RuntimeException(error, e);
        }
    }

    /* (non-Javadoc)
     * @see com.mysema.mailservice.MailService#sendMessage(java.lang.String, java.lang.String, java.lang.String, java.util.Map)
     */
    public final void sendMessage(String recipient, String subject, String template, Map<String,?> model) {
        sendMessage(new String[]{recipient}, subject, populateTemplate(template, model));
    }

    /* (non-Javadoc)
     * @see com.mysema.mailservice.MailService#sendMessage(java.lang.String[], java.lang.String, java.lang.String, java.util.Map)
     */
    public final void sendMessage(String[] recipients, String subject, String template, Map<String,?> model) {
        sendMessage(recipients, subject, populateTemplate(template, model));
    }
    
    public void sendHTMLMessage(String recipient, String subject,
            String template, Map<String, ?> model) {
        sendHTMLMessage(new String[]{recipient}, subject, populateTemplate(template, model));
        
    }

    public void sendHTMLMessage(String[] recipients, String subject,
            String template, Map<String, ?> model) {
        sendHTMLMessage(recipients, subject, populateTemplate(template, model));
        
    }
    
    protected String populateTemplate(String template, Map<String,?> model){
        throw new UnsupportedOperationException();
    }

    public void setMailSender(JavaMailSender mailSender) {
        this.mailSender = mailSender;
    }

    public void setMessageTemplate(SimpleMailMessage messageTemplate) {
        this.messageTemplate = messageTemplate;
    }




}
